/*****************************************************************************
 * Copyright (C) 2012 x264 project
 *
 * Authors: Anton Mitrofanov <BugMaster@narod.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111, USA.
 *****************************************************************************/

#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#if _WIN32
#include <sys/types.h>
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif
#include <time.h>
#include <signal.h>

#include <x264.h>

/* Ctrl-C handler */
static volatile int b_ctrl_c = 0;
static void sigint_handler( int a )
{
    b_ctrl_c = 1;
}

#define RETURN_IF_ERR( cond, ... )\
if( cond )\
{\
    fprintf( stderr, __VA_ARGS__ );\
    return -1;\
}

#define FAIL_IF_ERR( cond, ... )\
if( cond )\
{\
    fprintf( stderr, __VA_ARGS__ );\
    goto fail;\
}

int64_t get_mdate( void )
{
#if _WIN32
    struct timeb tb;
    ftime( &tb );
    return ((int64_t)tb.time * 1000 + (int64_t)tb.millitm) * 1000;
#else
    struct timeval tv_date;
    gettimeofday( &tv_date, NULL );
    return (int64_t)tv_date.tv_sec * 1000000 + (int64_t)tv_date.tv_usec;
#endif
}

#define FRAMES_MAX 256

int main( int argc, char **argv )
{
    x264_param_t param;
    x264_picture_t pic_in, pic_out;
    x264_nal_t *nal;
    int i_nal, frames_num, flush_frames;
    x264_t *h = NULL;

    srand( get_mdate() );

    RETURN_IF_ERR( x264_param_default_preset( &param, NULL, NULL ), "x264_param_default_preset failed\n" )
    param.i_csp    = X264_CSP_I420;
    param.i_width  = 1280;
    param.i_height = 720;

    RETURN_IF_ERR( x264_picture_alloc( &pic_in, param.i_csp, param.i_width, param.i_height ), "x264_picture_alloc failed\n" )

    h = x264_encoder_open( &param );
    FAIL_IF_ERR( !h, "x264_encoder_open failed\n" )
    pic_in.i_pts = 0;
    frames_num = rand() % FRAMES_MAX;
    flush_frames = rand() & 1;

    /* Control-C handler */
    signal( SIGINT, sigint_handler );

    while( !b_ctrl_c )
    {
        if( pic_in.i_pts >= frames_num )
        {
            while( flush_frames && !b_ctrl_c && x264_encoder_delayed_frames( h ) )
                FAIL_IF_ERR( x264_encoder_encode( h, &nal, &i_nal, NULL, &pic_out ) < 0, "x264_encoder_encode failed (flush)\n" )
            x264_encoder_close( h );
            h = x264_encoder_open( &param );
            FAIL_IF_ERR( !h, "x264_encoder_open failed\n" )
            pic_in.i_pts = 0;
            frames_num = rand() % FRAMES_MAX;
            flush_frames = rand() & 1;
            continue;
        }
        /* FIXME fill pic_in data here */
        FAIL_IF_ERR( x264_encoder_encode( h, &nal, &i_nal, &pic_in, &pic_out ) < 0, "x264_encoder_encode failed\n" )
        pic_in.i_pts++;
    }

    x264_encoder_close( h );
    x264_picture_clean( &pic_in );
    return 0;

fail:
    if( h )
        x264_encoder_close( h );
    x264_picture_clean( &pic_in );
    return -1;
}